
/*******************************************************************
*    8-bit PIC SchmartBoard workshop LCD temperature display       *
*    8-07-09                                                       *        
*    Copyright SchmartBoard 2009                                  *
********************************************************************
* The program will display the temperature                     	   *
*                   											   *
* MPLAB IDE v8.20                                                  *
* HI-TECH C PRO for the PIC10-12-16 MCU Family V9.60PL5 Lite-mode  *
*                                                                  *
* Controller: PIC16F886                                            * 
* Serial Digital Thermal Sensor: TC74A0-5.0VCT                     *
* LCD Moudle: LCM-S01602DTR+M (HD44780)		                       *
*																   *														   *
*   Files required:												   *
*																   *		
*                   main.c										   *
*					cnfig886a.h (Set the Configuration Word)       *
*                   MID_LCD.c                                      *
*					MID_LCD.h   								   *
*                   pic.h        (Hi-Tech file)					   *
*																   *
********************************************************************
*																   *
*   Notes:														   *
*																   *
*   Device Fosc -> 4.00MHz (Internal OSC)           			   *
*																   *	
*******************************************************************/

#include <pic.h>				// processor if/def file
#include "config886a.h"
#include "mid_lcd.h"


//**********************************************
// **** Check config886a.h for the configuration
// **** Establish PIC16F886 Configuration Word
//***********************************************
__CONFIG  ( DEBUG_OFF & LVP_OFF  & FCM_ON & IESO_OFF & BOREN_OFF & CPD_OFF & CP_OFF & MCLRE_ON & PWRTE_ON & WDT_OFF & INTOSC_OSC );
            



//**********************************
//* Var Declaration
//**********************************
const char LCD_Msg1[]="SchmartBoard PIC"; 
const char LCD_Msg2[]="T:     F        "; 
unsigned char count     = 0;
unsigned char digi0     = 0;
unsigned char digi1     = 0;
unsigned char digi2     = 0;
unsigned char Temp      = 0;
unsigned int  iTemp     = 0 ;
unsigned long iTemp0    = 0 ;
unsigned char iTemp1    = 0 ;
bit           sign      = 0 ;
unsigned int  div_count = 0;
unsigned int  mul_count = 9; 
unsigned int  mul_temp  = 0;


/*****************************
    INITIALIZE FUNCTIONS
*****************************/
void Init_System(void)
{
 // 	
}

void Init_I2c(void)
{
    TRISC3=1;           // set SCL and SDA pins as inputs
    TRISC4=1;
    SSPCON  = 0x38;      // set I2C master mode
    SSPCON2 = 0x00;
    SSPADD = 39;        // 50k at 8Mhz clock, I2C Master mode, clock = OSC/4 (SSPADD +1)
    CKE=1;              // use I2C levels      worked also with '0'
    SMP=1;              // disable slew rate control  worked also with '0'
    SSPIF=0;            // clear SSPIF interrupt flag
    BCLIF=0;            // clear bus collision flag
}

void Read_TC74_I2c(void)
{
 SEN=1;
 while(!SSPIF){}
 SSPBUF = 0B10010000;   //TC74(TC74A0-5.0VCT) default address 1001000 plus write bit 0, check TC74 spec
 SSPIF=0;
 while(!SSPIF){}
 #asm
 nop
 nop
 nop
 #endasm 
 SSPBUF = 0x00;         //command read temp
 SSPIF=0;
 while(!SSPIF){}
 #asm
 nop
 nop
 nop
 #endasm 
 RSEN=1;
 SSPIF=0;
 while(!SSPIF){}
 SSPBUF = 0B10010001; //TC74 default address 1001000 plus read bit 1
 SSPIF=0;
 while(!SSPIF){}
 RCEN=1;
 SSPIF=0;
 while(!SSPIF){}
 ACKEN=1;
 SSPIF=0;
 while(!SSPIF){}
 SSPIF=0;
 PEN=1;
 while(!SSPIF){}
 SSPIF=0;
}


//***************************************
//*             Program Main ( )		*
//***************************************
void main(void)
{
	Init_System();
	OpenLCD();
 	LCD_Set_Cursor(0,0);
	putrsLCD(LCD_Msg1);
	LCD_Set_Cursor(0,1);
	putrsLCD(LCD_Msg2);
    LCD_Set_Cursor(6,1);
    WriteDataLCD(0xDF);    //SACII code 0xDF for symbol
    Init_I2c();

	while(1)
	{
     Read_TC74_I2c();
     Temp= SSPBUF;    //Temp data from TC74
     sign = 0; 


     ///*** Celsius to Fahrenheit      F = (C*9/5) + 32
     if((Temp&0x80)==0x80)
      {
       sign = 1;
       Temp = (~Temp) + 0x01;  //2's complement  
      }
      iTemp = Temp;
      mul_temp = iTemp;
      for(mul_count = 9;mul_count>1;mul_count--)   //multiplied by 9
        {
         iTemp=iTemp+mul_temp;
        }
      div_count = 0;
      if(iTemp>0x05)                         //divided by 5
       {
        while(iTemp>0x05)
             {
              iTemp=iTemp-0x05;
              div_count++;
             }
       }
      if(iTemp>0x02)                        // round off 
       {
        div_count=div_count+0x01;
       }
      iTemp= div_count; 
      if(~sign)
       {
        iTemp=iTemp+32;
       }        
      if(sign)
       {
        if(iTemp<32)
         {
          iTemp= 32 - iTemp;
          sign = 0;
         }
        else
         {
          iTemp=iTemp - 32;
         }
       }



       ///*** 16 bit to BCD 
       count = 16;       
       iTemp0=iTemp;
       iTemp1=0x00;
       while(count>0)
         {
          if((iTemp0&0x0F0000)>0x040000)
            {
             iTemp0=iTemp0+0x030000;
             } 
          if((iTemp0&0xF00000)>0x400000)
            {
             iTemp0=iTemp0+0x300000;
            } 
          if((iTemp0&0x0F000000)>0x04000000)
            {
              iTemp0=iTemp0+0x03000000;
            }
          if((iTemp0&0x0F0000000)>0x040000000)
            {
             iTemp0=iTemp0+0x030000000;
            }
           if((iTemp0&0x80000000)==0x80000000)   
            {
              iTemp0 =iTemp0  << 1;  
              iTemp1=iTemp1<< 1;
              iTemp1=iTemp1+1;
            }      
           else
              {
               iTemp0 =iTemp0  << 1;
               iTemp1=iTemp1 << 1;
              }        
               count = --count ;
          }  


     ///*** write BCD to LCD
     LCD_Set_Cursor(2,1); 
     WriteDataLCD(0x20);
     if(sign)
       {
        LCD_Set_Cursor(2,1);
        WriteDataLCD(0x2D);
        }
     LCD_Set_Cursor(3,1);   
     digi0=((iTemp0&0x0f000000)>>24)+ 0x30;
     WriteDataLCD(digi0);
     digi1=((iTemp0&0x0f00000)>>20)+ 0x30;
     WriteDataLCD(digi1);
     digi2=((iTemp0&0x0f0000)>>16) + 0x30;  
     WriteDataLCD(digi2) ;
    }	
}


